<xsl:stylesheet
  version="2.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xpath-default-namespace="http://www.w3.org/1999/xhtml"
  exclude-result-prefixes="xs">
  <!-- The xmlns:xs declaration is required for typing variables. But we
       do not want this declaration to be propagated to the output
       document, so we exclude it from the result. -->

  <xsl:output indent="yes" doctype-system="movies.dtd" />

  <xsl:template match="/">
    <movies>
      <xsl:for-each select="//@href">
        <movie><xsl:apply-templates select="document(.)/html" /></movie>
        <!-- Since there is no template definition for html, the default
             template is used and the document is browsed until one of
             the template is fired. -->
      </xsl:for-each>
    </movies>
  </xsl:template>

  <xsl:template match="span[@class='title-extra']">
    <title><xsl:value-of select="normalize-space(text())" /></title>
    <!-- normalize-space() often needs to be used to get rid of
         extraneous whitespace coming from the HTML source. -->
  </xsl:template>
  
  <xsl:template match="h4[contains(.,'Director:')]">
    <!-- Here, the template is fired not on the element containing
         portion of text that needs to be retrieved, but on a more
         identifiable element. -->
    <director>
      <xsl:value-of select="normalize-space(following-sibling::a/text())" />
    </director>
  </xsl:template>
  
  <xsl:template match="h4[contains(.,'Release Date:')]">
    <released>
      <xsl:analyze-string
        regex="^\s+(\d+)\s+([a-zA-Z]+)\s+(\d+)" 
        select="following-sibling::text()[1]">
        <!-- Regular expression features of XSLT 2.0 greatly facilitate
             parsing text. In XSLT 1.0, we would need to use string
             manipulation functions (substring, substring-before,
             normalize-space, etc.). -->
        <xsl:matching-substring>
          <day><xsl:value-of select="regex-group(1)" /></day>
          <month><xsl:value-of select="regex-group(2)" /></month>
          <year><xsl:value-of select="regex-group(3)" /></year>
        </xsl:matching-substring>
      </xsl:analyze-string>
    </released>
  </xsl:template>

  <xsl:template match="table[@class='cast_list']">
    <cast>
      <xsl:apply-templates select=".//tr" />
    </cast>
  </xsl:template>

  <xsl:template match="table[@class='cast_list']/tr[td/@class='character']">
    <xsl:variable name="character" as="xs:string">
      <xsl:value-of 
        select="td[@class='character']//text()[normalize-space()!='']" />
    </xsl:variable>
    <!-- This intermediate variable is used to store as a string (observe
         the xs:string annotation) the concatenated text nodes. -->
    <actor as="{normalize-space($character)}">
      <xsl:value-of select="normalize-space(td[@class='name']//text())" />
    </actor>
  </xsl:template>

  <xsl:template match="text()" />
  <!-- We override the default template for text() nodes to remove from
       the output everything that is not specifically requested when
       browsing the DOM tree. -->
</xsl:stylesheet>
